import * as vue_demi from 'vue-demi';
import { VNode, Ref, AppContext, Component, ExtractPropTypes, App } from 'vue-demi';

interface OverlayMeta {
    /** the notification cancel, modify visible, and destroy it after the animation ends */
    cancel: Function;
    /** the notification confirm, modify visible, and destroy it after the animation ends */
    confirm: Function;
    /** destroy the current instance (immediately) */
    vanish: Function;
    /** vnode the VNode of the current injection layer */
    vnode?: VNode;
    /** visible control popup display and hide */
    visible: Ref<boolean>;
    /** use in template */
    isTemplate?: boolean;
}

interface UseOverlayMetaOptions {
    /** animation duration to avoid premature destruction of components */
    animation?: number;
    /** whether to set visible to true immediately */
    immediate?: boolean;
    /**
     * v-model fields used by template
     *
     * @default 'visible'
     */
    model?: string;
    /**
     * cancel event name used by the template
     *
     * @default 'cancel'
     */
    cancel?: string;
    /**
     * confirm event name used by the template
     *
     * @default 'confirm'
     */
    confirm?: string;
    /**
     * whether to automatically handle components based on visible and animation
     *
     * @default true
     */
    automatic?: boolean;
}
/**
 * get overlay layer meta information
 * @function cancel  the notification cancel, modify visible, and destroy it after the animation ends
 * @function confirm the notification confirm, modify visible, and destroy it after the animation ends
 * @function vanish destroy the current instance (immediately)
 * @field vnode the VNode of the current injection layer
 * @field visible control popup display and hide
 * @returns
 */
declare function useOverlayMeta(options?: UseOverlayMetaOptions): OverlayMeta;

interface MountOverlayOptions {
    /** the dom node to mount at render time */
    root?: HTMLElement;
    /** current app context */
    appContext?: AppContext;
}

declare type ImperativePromise<T = any> = Promise<T> & {
    cancel: Function;
    confirm: Function;
};
declare type ExtractInferTypes<Props> = Props extends ExtractPropTypes<infer E> ? E : ExtractPropTypes<Props>;
declare type ImperativeOverlay<Params, Resolved> = (props?: ExtractInferTypes<Params>, options?: MountOverlayOptions) => ImperativePromise<Resolved>;
/**
 * Create imperative overlay
 * @param component Component
 */
declare function createOverlay<Params = any, Resolved = void>(component: Component): ImperativeOverlay<Params, Resolved>;

interface ExecuteOverlayOptions<P = any> extends MountOverlayOptions {
    props?: P;
}
/**
 * Execute overlay component
 * @param component Component
 * @param options mount options and props
 */
declare function executeOverlay<P = any, R = any>(component: Component, options?: ExecuteOverlayOptions<P>): ImperativePromise<R>;

declare const install: (app: App) => void;
declare const unoverlay: {
    install: (app: App) => void;
};

declare const OverlayProvider: vue_demi.DefineComponent<{}, () => vue_demi.VNode<vue_demi.RendererNode, vue_demi.RendererElement, {
    [key: string]: any;
}>[] | undefined, {}, {}, {}, vue_demi.ComponentOptionsMixin, vue_demi.ComponentOptionsMixin, vue_demi.EmitsOptions, string, vue_demi.VNodeProps & vue_demi.AllowedComponentProps & vue_demi.ComponentCustomProps, Readonly<vue_demi.ExtractPropTypes<{}>>, {}>;

export { ImperativeOverlay, OverlayProvider, createOverlay, unoverlay as default, executeOverlay, install, useOverlayMeta };
