"use strict";

// src/index.ts
import { defineComponent as defineComponent2, getCurrentInstance as getCurrentInstance2 } from "vue-demi";

// src/internal/index.ts
var context = {
  appContext: null
};
var OverlayMetaKey = Symbol("__imperative_overlay_key");

// src/meta.ts
import { getCurrentInstance, inject, onMounted, provide as provide2, ref as ref2, watch } from "vue-demi";
import { useVModel } from "@vueuse/core";

// src/transform.ts
import { provide, ref } from "vue-demi";

// src/helper/render.ts
import { createVNode, defineComponent, h, render } from "vue-demi";
var renderInstance = (component, props, options = {}) => {
  const vanish = () => {
    render(null, container);
  };
  const Provider = defineComponent({
    name: `${component.name}OverlayProvider`,
    setup() {
      var _a;
      (_a = options.provide) == null ? void 0 : _a.call(options, vnode, vanish);
    },
    render() {
      return h(component, props);
    }
  });
  const container = document.createElement("div");
  container.className = "overlay-container";
  const vnode = createVNode(Provider, props);
  vnode.appContext = options.appContext || context.appContext;
  render(vnode, container);
  if (container.firstElementChild) {
    const root = options.root || document.body;
    root.append(container.firstElementChild);
  }
  return { vanish, vnode };
};

// src/transform.ts
function createOverlay(component) {
  const executor = (props, promiser, mountOptions) => {
    const { reject, resolve, promise } = promiser;
    renderInstance(component, props, {
      appContext: mountOptions == null ? void 0 : mountOptions.appContext,
      root: mountOptions == null ? void 0 : mountOptions.root,
      provide: (vnode, _vanish) => {
        const visible = ref(false);
        function cancel(value) {
          reject == null ? void 0 : reject(value);
          visible.value = false;
        }
        function confirm(value) {
          resolve == null ? void 0 : resolve(value);
          visible.value = false;
        }
        function vanish() {
          _vanish == null ? void 0 : _vanish();
          reject == null ? void 0 : reject();
        }
        promise.confirm = confirm;
        promise.cancel = cancel;
        provide(OverlayMetaKey, {
          cancel,
          confirm,
          vanish,
          visible,
          vnode
        });
      }
    });
  };
  const caller = (props, options) => {
    const promiser = createPromiser();
    const notAllowedError = function() {
      throw new Error("overlay - Error: It is not allowed to call confirm and cancel externally immediately, please wait for the component to render");
    };
    promiser.promise.confirm = notAllowedError;
    promiser.promise.cancel = notAllowedError;
    executor(props, promiser, options);
    return promiser.promise;
  };
  return caller;
}
function createPromiser() {
  let resolve;
  let reject;
  const promise = new Promise((_resolve, _reject) => {
    resolve = _resolve;
    reject = _reject;
  });
  return { promise, reject, resolve };
}

// src/utils.ts
function executeOverlay(component, options) {
  return createOverlay(component)(options == null ? void 0 : options.props, options);
}
function delay(milliseconds) {
  return new Promise((resolve) => setTimeout(resolve, milliseconds));
}
var noop = () => {
};

// src/meta.ts
function useOverlayMeta(options = {}) {
  const { animation = 0, immediate = true, model = "visible", automatic = true } = options;
  const defaultMeta = getTemplateMeta(model, options);
  const meta = inject(OverlayMetaKey, defaultMeta) || defaultMeta;
  if (!meta.isTemplate && automatic) {
    watch(meta.visible, async () => {
      var _a;
      if (meta.visible.value)
        return void 0;
      if (animation > 0)
        await delay(animation);
      (_a = meta.vanish) == null ? void 0 : _a.call(meta);
    });
  }
  if (immediate)
    onMounted(() => meta.visible.value = true);
  provide2(OverlayMetaKey, null);
  return meta;
}
function getTemplateMeta(model, options = {}) {
  const instance = getCurrentInstance();
  const visible = instance ? useVModel(instance.props, model) : ref2(false);
  const cancel = (value) => {
    visible.value = false;
    instance == null ? void 0 : instance.emit(options.cancel || "cancel", value);
  };
  const confirm = (value) => {
    visible.value = false;
    instance == null ? void 0 : instance.emit(options.confirm || "confirm", value);
  };
  return {
    cancel,
    confirm,
    vanish: noop,
    visible,
    isTemplate: true
  };
}

// src/index.ts
var install = (app) => {
  context.appContext = app._context;
};
var unoverlay = { install };
var OverlayProvider = defineComponent2({
  setup(_, { slots }) {
    const { appContext } = getCurrentInstance2();
    context.appContext = appContext;
    return () => {
      var _a;
      return (_a = slots.default) == null ? void 0 : _a.call(slots);
    };
  }
});
var src_default = unoverlay;
export {
  OverlayProvider,
  createOverlay,
  src_default as default,
  executeOverlay,
  install,
  useOverlayMeta
};
//# sourceMappingURL=index.esm.js.map
